; OKPRINT has the printer routines for the various printers
;
DSEG    SEGMENT PARA PUBLIC 'DATA'
    ASSUME CS:CSEG,DS:DSEG,ES:DSEG
extrn   work_seg:word,printer:byte,pdensity:byte
extrn   p_xor:byte,vwid:word,debug:byte,lpt:word,tree31x:word
extrn   strg:byte,handle:word,height_pix:word,width_bytes:word
extrn   vwid:word,srow:word,scol:word,left_margin:word
extrn   curx:word,cury:word,baud:byte,parity:byte
extrn   p_pass:byte
segmentsize equ 8000h
print macro string
    mov   left_margin,64
    mov   curx,64
    mov   cury,64
    call  blip
    lea   dx,string
    call  print_string
    endm
;
; LOCAL DATA
;
;
err5   db   "ERROR OPENING FILE",13,10
    db   "PRESS ANY KEY TO CONTINUE",0
err6   db   "YOU MUST SELECT A VALID RANGE BEFORE PRINTING",13,10
    db   "PRESS ANY KEY TO CONTINUE",0
;
this_pass db  0
;
delimiter db ','
fbuffer  db  0    ; used to write file
ptarget   db  0
perr    db  1
crlf  db 13,10,255
extrn lines_per_byte:word
;
; centronix printer
;
cent_init  db 27,'%'
cent_mode  db '0',0ffh
cent_line db  13,0ffh
cent_stop  db 50,0ffh
;
; Toshiba dot matrix  24 pin
;
tosh_init  db   27,">",13,27,"L07",0ffh
tosh_line  db   13,27
tosh_den   db   ';'
tosh_count db  '0000',0ffh
;
; ESC A printers
;
esc3    db  27,'3',24,0ffh
escA9   db  27,'A',8,0ffh
escm    db  13,27,'*'
escm_density db 0
escm_lines  dw 576
    db  0ffh
esck    db  13,27
esck_density db 'K'
esck_lines  dw 576
    db  0ffh
;
; NCR escape sequences
;
ncr_den   db  27,'T16',13,27
ncr_den_type db 'N',13,255
ncr_gr    db    13,27,'S'
ncr_dec   db  '0001',255
;
;  CANON LBP data
;
canon  db 27,';',9bh,0ffh
;
; RICOH data
;
ricoh  db  27,18,'G3,',0ffh
;
;
;HP LaserJet escape sequences
;
hp_init  db 13,27,'*t'
hp_res   db '300R',0ffh
hp_start db 27,'*r1A',0ffh
hp_transfer db 27,'*b'
hp_count db '0000W',0ffh
hp_end   db 27,'*rB',0ffh
hp_port  db  27,'&l0O',0ffh
hp_land  db  27,'&l1O',27,'&a7632H',0ffh
;
;
; OKI sequences
;
OKI_start  db 28,3,0ffh
oki_crlf    db 3,14,0ffh
oki_end    db 3,2,30,0ffh
;
;
;  gemini codes
;
esc33   db  1bh,33h,10h,0ffh
gem_line db  13,27
escg_density db 75
escg_lines  dw 576
    db  0ffh
;
;
; NEC Pinwriter codes
nec_line db 13,28,'Z'
nec_width dw 0
  db 0ffh
;
;
; Proprinter 24
p24_init  db 0ffh
p24_line  db 27,74,28,13,27,91,103
p24_count dw 0
p24_density db 0  ; 8,9,11 or 12
    db 0ffh
;
; lq 24
lq_init  db 27,51,24,0ffh
lq_line  db 13,10,27,'*'
lq_density db 0  ; 32,33,38,39,40
lq_count dw 0
    db 0ffh
;
;
;
;
DSEG   ENDS
;
CSEG    SEGMENT PARA PUBLIC 'CODE'
    ASSUME CS:CSEG,DS:DSEG,ES:DSEG
extrn   present:near,quit:near,get_next_seg:near,get_curr_seg:near
extrn   reset_segs:near,print_string:near
extrn   create_file:near,write_file:near,close_file:near
extrn   list_filep:near,getkey:near,do_wait:near,next_wait:near
extrn   do_tree:near,blip:near,blop:near,calc_file_size:near
extrn   goback:near
;
;
; JUMP TABLES
;
printprocs dw  do_bj80,do_hp,do_hp,do_bj80,do_ncr,do_hp,do_tosh
    dw  do_bj80,do_ncr,do_ncr,do_bj80,do_nec,do_p24,do_p24
;
printinits dw  init_bj80,init_hp,init_ricoh,init_esck,init_ncr
    dw  init_canon,init_tosh,init_gem,init_OKI,init_cent
    dw  init_bj80,init_esck,init_p24,init_lq
;
printlines dw  line_bj80,line_hp,ricoh_ret,line_esck,line_ncr,ricoh_ret
    dw  line_tosh,line_gem,line_OKI,line_cent,line_esck,line_nec
    dw  line_p24,line_lq
;
endline    dw  pr_crlf,p_ret,p_ret,pr_crlf,pr_crlf,p_ret,pr_crlf
    dw  pr_crlf,pr_crlf,pr_crlf,pr_crlf,pr_crlf,p_ret,p_ret,p_ret
    dw  p_ret,p_ret,p_ret,p_ret,p_ret,p_ret,p_ret,p_ret,p_ret,p_ret
;
end_procs  dw  pr_crlf,end_hp,p_ret,pr_crlf,pr_crlf
    dw  p_ret,pr_crlf,pr_crlf,end_oki,end_cent,pr_crlf,pr_crlf
    dw  p_ret,pr_crlf
;
;
;==========================================;
;    ;
;   The main print procedure is here    ;
;    ;
;==========================================;
;
;
printer_procs  proc  near
;
;
; check for debug
;
    call reset_segs
    cmp  debug,8
    jne  no_debug
    int  3    ; debug break
no_debug:
; we must calculate the file size, best way is to use the file routines
;
    call  calc_file_size   ; figures out the width and height
    cmp   height_pix,0
    jle   bad_range
    cmp   width_bytes,0
    jle   bad_range
    jmp   good_range
bad_range:
    print err6
    call getkey
    call present
    ret
;
; The DS register will always be set correctly so that we can do away
; the this CX and DX crap.
; The ES register is altered, but we will reset_segs when we leave
;
good_range:
; call the init stuff
    mov   bl,printer
    mov   bh,0
    shl   bx,1
    mov   bp,width_bytes ; hoepfully every routine will figeure out
    mov   cx,height_pix  ; this by itself, but until then
;
;  INIT
    call  word ptr cs:printinits[bx] ; prints the printer reset info, etc
;
;
; have to figure out the start positions of the thing
; this calculates the start position
;
    mov   dx,0
    mov   ax,segmentsize
    div   vwid
; ax has the lines in a segment
    mov   cx,ax
    mov   ax,srow  ;starting row
    mov   dx,0   ; is zero anyway, but to be sure
    div   cx
; ax has the number of starting segment
; dx has the remainder line
    mov   work_seg,ax
    mov   ax,dx
    mul   vwid
    add   ax,scol
    mov   di,ax    ; di has the starting offset within work_seg
;
; things are set up
;    cx has the number of rows
;    di has the starting offset to work with
;    bp has the number of columns
;
    mov   bp,width_bytes ; thowback to pcrgb conventions
; pin printers have to have the count of lines to work on adjusted
; have to adjust cx as per lines per byte
    mov   ax,height_pix
    mov   dx,0
    add   ax,lines_per_byte
    dec   ax
    div   lines_per_byte
; ax has the new line to print
    mov   dx,0
    mov   cx,ax
; check for double strike must shl for extra passes
    cmp   p_pass,0
    je    no_x_pass0
    shl   cx,1
no_x_pass0:
; now we can start off by setting the current segment in the pic
;
    call  get_curr_seg
    mov   es,ax
; make sure that we have at least than 1 guy in the CX register
    cmp   cx,1
    jg    prt_proc_p_loop
    mov   cx,1
prt_proc_p_loop:
;
; this is the loop that is executed once for each pass of the head
;
; up to here the stack is clean.
;
    push  cx   ; count of lines
    push  di   ; count of offset
; check to see if the hourglass is needed
    test  cx,7
    jnz  no_wait
    call  next_wait
no_wait:
;
;  LINE Call, this is executed to start a line
;
; here we send out the string to print a line escape character
    mov   bl,printer
    mov   bh,0
    shl   bx,1
    mov   bp,width_bytes
    push  di
    call  cs:printlines[bx] ; ends esc* or whatever
    pop   di
; this is where we loop through the print area
;
; before we get the segments messed up, save the work_seg
    mov  ax,work_seg
    push  ax
;
    mov   cx,width_bytes ; the count of bytes in one line
; in the stack is ax,di,cx
prt_proc_p_loop1:
; here is where we call the various printer procedures
    mov   bl,printer
    mov   bh,0
    shl   bx,1
    push  cx
    push  di
; here, es:di points to a byte which must be printed
    call  cs:printprocs[bx] ; gets a byte and prints it out
    pop   di
    pop   cx
    cmp   perr,0
    je    noerr_p
    call  reset_segs
    pop   ax
    pop   di
    pop   cx
    stc
    ret
noerr_p:
; the printprocs printed a byte, but it may have reset a segment
    pop   ax
    push  ax
    cmp   work_seg,ax
    je    ok_work_seg
    mov   work_seg,ax
    call  get_curr_seg
    mov   es,ax
ok_work_seg:
    inc   di   ; no need to check, we are on a single line
    loop  prt_proc_p_loop1
;
;
;
    pop   ax    ; pop off the work seg used above
    pop   di    ; old di
; here we compensate for sending multiple lines to printer
    cmp   p_pass,0
    je    no_x_pass
    xor   this_pass,1
    cmp   this_pass,1
    je    prt_proc_p_3
no_x_pass:
    mov   bl,printer
    mov   bh,0
    shl   bx,1
; here, es:di points to a byte which must be printed
    push  es
    push  di
    call  cs:endline[bx] ; performs end of line stuff
    pop   di
    pop   es
    mov   ax,lines_per_byte
    mul   vwid
; here check for the number of passes needed
    add   di,ax
    cmp   di,0
    jge    prt_proc_p_3
    sub    di,segmentsize
    call   get_next_seg
    mov    es,ax
prt_proc_p_3:
    pop   cx
    loop  prt_proc_p_loop_b
; finished the loop m now call the end procedure
    mov   bl,printer
    mov   bh,0
    shl   bx,1
    call  cs:end_procs[bx] ; performs ending tasks
    call  reset_segs
    call  present
    ret
prt_proc_p_loop_b:
    jmp   prt_proc_p_loop
p_ret:
    ret
pr_crlf:
    lea   si,crlf
    call  print_out
    ret
printer_procs endp
;
;
;
;
;====================================================;
;    ;
;  MISC PRINT FUNCTIONS    ;
;    ;
;====================================================;
;
;
; the following routines perform miscellaneous functions
; such as the actual printing of a byte, or CRLF or eject, etc
print_byte:
; save state
    push  ax
    push  bx
    push  cx
    push  dx
    push  bp
    push  si
    push  di
; test to see if a key has been printed
    mov   perr,0    ; perr has the error code if  there is one
    push  ax
check_again_key:
    mov   ah,1
    int   16h
    jz    no_key_pressed
    mov   ah,0
    int   16h   ; get the key
    cmp   al,27
    jne   check_again_key
    pop   ax
    mov   ah,1
    mov   perr,1
    jmp   bad_print
no_key_pressed:
    pop   ax
    cmp   ptarget,0
    je    print_byte_printer
; write a byte out to a file
    mov   fbuffer,al
    mov   cx,1
    lea   dx,fbuffer
    call  write_file
    jmp   ok_ah
print_byte_printer:
    mov    dx,lpt
    cmp    dx,3
    jge    print_byte_comm
    mov   cx,128
attempt_loop:
    mov   ah,0
    int    17h
    test  ah,25h
    jz    ok_ah
    loop  attempt_loop
;
    mov    perr,1
    jmp    bad_print
ok_ah:
    pop   di
    pop   si
    pop   bp
    pop   dx
    pop   cx
    pop   bx
    pop   ax
    ret
print_byte_comm:
    mov   ah,1
    sub   dx,3
    int   14h
    and   ah,80h
    cmp   ah,80h
    jne   ok_ah
    mov    perr,1
    jmp    bad_print
; this prints the thing out the serial port
;
;
;
print_out: ;prints a string pointed to by ds:si
    push  ax
    push  bx
    push  cx
    push  dx
    push  bp
    push  si
    push  di
;
;
print_out_loop:
    cld
    lodsb
    cmp   al,0ffh
    je    ps_return
    call  print_byte
    jmp  print_out_loop
ps_return:
    jmp   ok_ah
bad_print:
    call    blop
    call    blop
    call    blop
    call    blop
    call    blop
    call    goback
;
;
;  this is the set page called from the tree
;
dumb_proc proc near
set_page:
public set_page
    mov   al,12
    call  print_byte
    call  blip
    ret
dumb_proc  endp
;
;  this is the set_init called from the tree
;
set_init:
public set_init
    call  blip
    call  do_init
    ret
do_init:
; initializes the com port
    push  ax
    push  dx
    mov   ah,0
    mov   al,baud
    or    al,parity
    mov  dx,lpt
    clc
    sub  dx,3
    jc   bad_com_port
    int  14h
    pop   dx
    pop   ax
    ret
bad_com_port:
    call   blop
    pop   dx
    pop   ax
    ret
;
; set_lf is called from ttree
;
set_lf:
public set_lf
    mov   al,13
    call  print_byte
    mov   al,10
    call  print_byte
    call  blip
    ret
;
; do_space is called from ttree
;
do_space:
public do_space
    mov   al,20h
    call  print_byte
    call  blip
    ret
do_print:
; this is the main entry for GO on menu
public do_print
    push  bp
    push  di
    push  ds
    push  es
    call  reset_segs
    mov   this_pass,0
    call  do_wait
    call  printer_procs
    pop   es
    pop   ds
    pop   di
    pop   bp
    ret
;
;
; TREE entry for the the printer stuff is here
;
;
get_p_printer:
public get_p_printer
 ; this is for printing to a printer
    call  present
    mov   ptarget,0
    mov   bp,0
    lea   si,tree31x
    call do_tree
    ret

get_p_file:
public get_p_file
    call  reset_segs
    mov   ptarget,1
    push si
    call present
    call  list_filep
    cmp  al,27
    jne   g_p_f_1
    pop  si
    ret
g_p_f_1:
    cmp  strg,0
    jne  g_p_f_2
    pop  si
    jmp  get_p_file
g_p_f_2:
; open the file for output
    call create_file
    jnc  g_p_f_3
    cmp  handle,0
    jne  g_p_f_3
; bad file
; say bad file message
    print err5
    call getkey
    call present
    pop  si
    jmp get_p_file
g_p_f_3:
; have a file to print to
    call  present
    mov   bp,0
    lea   si,tree31x
    call do_tree
    call  close_file
    pop  si
    ret

;=========================================;
;    ;
;   TRANSFER "DO" ROUTINES    ;
;    ;
;=========================================;
;
;  HP do data transfer
;
do_hp:
    mov    al,es:[di]
; the byte to be printed is in al
    cmp    p_xor,1
    jne    no_hp_xor
    xor    al,0ffh
no_hp_xor:
    call    print_byte
    ret

do_tosh:
; here, es:di points to a byte which must be printed
    mov    cx,8    ; eight bits wide
tprint_loop:
    push    cx
    mov    ax,work_seg
    push    ax
    mov    si,di
    mov    cx,4    ; 4 sets of 6 deep
tloop1:
    push    cx
    mov    cx,6    ; 6 bits deep
    mov    ax,0
tloop3:
    push    cx
    shl    ah,1
    mov    al,es:[si]
    clc
    shl    al,1
    jnc    tno_carry
    or    ax,0101h
tno_carry:
    mov    es:[si],al
    add    si,vwid
;    check si range
    cmp   si,0
    jge    tlok1
    push   ax
    call   get_next_seg
    mov  es,ax
    pop   ax
    sub    si,segmentsize
tlok1:
    pop    cx
    loop    tloop3
; the byte to be printed is in ah
    mov    al,ah
    cmp    p_xor,1
    jne    no_t_xor
    xor    al,0ffh
no_t_xor:
    call    print_byte
    pop    cx
    loop    tloop1
    pop    ax
    mov    work_seg,ax
    call    get_curr_seg
    mov    es,ax
    pop    cx
    loop    tprint_loop
    ret

do_bj80:
; here, es:di points to a byte which must be printed
    mov    cx,8
print_loop:
    push    cx
    mov    ax,work_seg
    push    ax
    mov    si,di
    mov    cx,8
    mov    ax,0
ploop3:
    push    cx
    shl    ah,1
    mov    al,es:[si]
    clc
    shl    al,1
    jnc    no_carry
    or    ax,0101h
no_carry:
    mov    es:[si],al
    add    si,vwid
;    check si range
    cmp   si,0
    jge    plok1
    push   ax
    call   get_next_seg
    mov    es,ax
    sub    si,segmentsize
    pop    ax
plok1:
    pop    cx
    loop    ploop3
; the byte to be printed is in ah
    mov    al,ah
    cmp    p_xor,1
    jne    no_p_xor
    xor    al,0ffh
no_p_xor:
    call    print_byte
    pop    ax
    mov    work_seg,ax
    call    get_curr_seg
    mov    es,ax
    pop    cx
    loop    print_loop
    ret

do_ncr:
; here, es:di points to a byte which must be printed
; the MSB is the opposite of epson
; ncr do it backwards
; OKI do it backwards too (perverts)
    mov    ax,0
    mov    cx,8
pncr_loop:
    push    cx
    mov    ax,work_seg
    push    ax
    mov    si,di
    mov    cx,8
    mov    ax,0
pncrloop3:
    push    cx
    clc
    shr    ah,1
    mov    al,es:[si]
    clc
    rcl    al,1
    jnc    no_carryn
    or    ax,8001h
no_carryn:
    mov    es:[si],al
    add    si,vwid
;    check si range
    cmp   si,0
    jge    pncrlok1
    push    ax
    call   get_next_seg
    mov    es,ax
    pop    ax
    sub    si,segmentsize
pncrlok1:
    pop    cx
    loop    pncrloop3
; the byte to be pncred is in ah
    mov    al,ah
    cmp    p_xor,1
    jne    no_n_xor
    xor    al,0ffh
no_n_xor:
; check to see if we are a centronix
    cmp    printer,9
    jne    not_cent
    and    al,00111111b
    add    al,20h
not_cent:
; check to see if we are a oki
    cmp    printer,8
    jne    not_oki
    cmp    al,3
    jne    not_oki
    push    ax
    call    print_byte
    pop    ax
not_oki:
    call    print_byte
    pop    ax
    mov    work_seg,ax
    call    get_curr_seg
    mov    es,ax
    pop    cx
    loop    pncr_loop
    ret

do_p24:
; here, es:di points to a byte which must be printed
    mov    cx,8    ; eight bits wide
p24print_loop:
    push    cx
    mov    ax,work_seg
    push    ax
    mov    si,di
    mov    cx,3    ; 3 sets of 8 deep  msb
p24loop1:
    push    cx
    mov    cx,8    ; 8 bits deep
    mov    ax,0
p24loop3:
    push    cx
    shl    ah,1
    mov    al,es:[si]
    clc
    shl    al,1
    jnc    p24no_carry
    or    ax,0101h
p24no_carry:
    mov    es:[si],al
    add    si,vwid
;    check si range
    cmp   si,0
    jge    p24lok1
    push   ax
    call   get_next_seg
    mov  es,ax
    pop   ax
    sub    si,segmentsize
p24lok1:
    pop    cx
    loop    p24loop3
; the byte to be printed is in ah
    mov    al,ah
    cmp    p_xor,1
    jne    no_p24_xor
    xor    al,0ffh
no_p24_xor:
    call    print_byte
    pop    cx
    loop    p24loop1
    pop    ax
    mov    work_seg,ax
    call    get_curr_seg
    mov    es,ax
    pop    cx
    loop    p24print_loop
    ret
;
;
do_nec:
; here, es:di points to a byte which must be printed
    mov    cx,8    ; eight bits wide
necprint_loop:
    push    cx
    mov    ax,work_seg
    push    ax
    mov    si,di
    mov    cx,3    ; 3 sets of 8 deep  msb
necloop1:
    push    cx
    mov    cx,8    ; 6 bits deep
    mov    ax,0
necloop3:
    push    cx
    shl    ah,1
    mov    al,es:[si]
    clc
    shl    al,1
    jnc    necno_carry
    or    ax,0101h
necno_carry:
    mov    es:[si],al
    add    si,vwid
;    check si range
    cmp   si,0
    jge    neclok1
    push   ax
    call   get_next_seg
    mov  es,ax
    pop   ax
    sub    si,segmentsize
neclok1:
    pop    cx
    loop    necloop3
; the byte to be printed is in ah
    mov    al,ah
    cmp    p_xor,1
    jne    no_nec_xor
    xor    al,0ffh
no_nec_xor:
    call    print_byte
    pop    cx
    loop    necloop1
    pop    ax
    mov    work_seg,ax
    call    get_curr_seg
    mov    es,ax
    pop    cx
    loop    necprint_loop
    ret

;
;==================================================;
;    ;
;   INITIALIZATION ROUTINES    ;
;    ;
;==================================================;
;
;
init_cent:
; here  have to print cent_init
    push    si
    push    di
    mov    cent_mode,'0'
    cmp    pdensity,0
    je    do_initc
    mov    cent_mode,'1'
    cmp    pdensity,1
    je    do_initc
    mov    cent_mode,'2'
do_initc:
    lea    si,cent_init
    call    print_out
    pop   di
    pop   si
    ret
;
;
;
;
init_gem:
; here  have to print esc,33,10
    push    si
    push    di
    lea    si,esc33
    call    print_out
; we have to set up the columns to print out
; and the density
; bp has the columns to print
    mov   ax,width_bytes
    mov   cx,3
    shl   ax,cl
    mov   escg_lines,ax
; now density
    mov   al,pdensity
    mov   escm_density,al
;    print mess1
; for ESC_K printers
    mov   escg_density,75
    cmp   al,0
    je    done_densityg
    mov   escg_density,76
    cmp   al,1
    je    done_densityg
    mov   escg_density,121
    cmp   al,2
    je    done_densityg
    mov   escg_density,122
done_densityg:
    pop   di
    pop   si
    ret
;
;
;
;
;
; NCR sequence
;
init_ncr:
    push  si
    push  di
    mov   al,pdensity
    mov   ncr_den_type,'N'
    cmp   al,0
    je    don_n_den
    mov   ncr_den_type,'E'
    cmp   al,1
    je    don_n_den
    mov   ncr_den_type,'Q'
    cmp   al,2
    je    don_n_den
    mov   ncr_den_type,'P'
don_n_den:
    lea   si,ncr_den
    call  print_out
; bp has the number of bytes on a line
    mov    ax,width_bytes
    mov    cx,8
    mul    cx    ; multiply by 8
    call   convert_to_decimal
;    dh has 1's, dl has 10's, ah has 100's, al has 1000's
    lea   di,ncr_dec
    cld
    mov   ds:[di],ax
    inc  di
    inc  di
    mov  ax,dx
    mov   ds:[di],ax
    pop    di
    pop    si
    ret
;
;   INITIALIZE CANON
;
init_canon:
;cx has the length and bp has the width
;
; this sets up the one ricoh procedure and sends it out the printer
;
    push    si
    push    di
    mov    delimiter,';'
    lea    si,canon
    call    print_out
; now convert the parameters into decimal and print
; first get the width in dots
    mov    ax,width_bytes
    mul    height_pix
    call   send_num_comma   ;size
    mov    ax,width_bytes
    mov    dx,0
    call   send_num_comma   ;width
    mov    ax,75
    cmp    pdensity,0
    je    ok_canon_d
    mov    ax,100
    cmp    pdensity,1
    je    ok_canon_d
    mov    ax,150
    cmp    pdensity,2
    je    ok_canon_d
    mov    ax,300
ok_canon_d:
    mov    dx,0
    call   send_num_comma  ; resolution
    mov    al,'.'
    call   print_byte
    mov    al,'r'
    call   print_byte
    pop  di
    pop  si
    ret
;
;
;  RICOH sequence
;
;
init_ricoh:
;cx has the length and bp has the width
;
; this sets up the one ricoh procedure and sends it out the printer
;
    push    si
    push    di
    mov    delimiter,','
    lea    si,ricoh
    call    print_out
; now convert the parameters into decimal and print
; first get the width in dots
    mov    ax,width_bytes
    mov    dx,0
    mov    cx,8
    mul    cx
    call   send_num_comma   ;m2
    mov    ax,height_pix
    mov    dx,0
    call   send_num_comma   ;m3
    mov    al,pdensity
    inc    al
    or    al,30h
    cmp    al,34h
    jle    ok_ricoh_d
    mov    al,34h
ok_ricoh_d:
    call   print_byte   ; m4
    call   send_comma   ; m5
    call   send_comma   ; m6
    mov    al,'@'    ;m7
    call   print_byte
    pop    di
    pop    si
    ret
convert_to_decimal:
 ;converts number in ax to decimal and returns it in dx and ax
 ; returns in bx the high two digits if it is over 9999
    push   cx
    mov   cx,10000
    div   cx
    push  dx
    mov   cx,10
    div   cl
    or    ax,3030h
    mov   bx,ax
    pop   ax
    mov   dx,0
    mov   cx,100
    div    cx
    mov   cx,10
    div   cl
    or    ax,3030h
    xchg  ax,dx
    div   cl
    or    ax,3030h
    xchg  ax,dx
    pop   cx
    ret
;    dh has 1's, dl has 10's, ah has 100's, al has 1000's
;   bh has the 10,000's and bl has the 100,000's
send_num_comma:
; this takes the contents of dx:ax and puts it out the printer port
; as decimal and then sends a comma
    call   convert_to_decimal
;    bx,dx and ax have the stuff
    push  dx    ;1
    push  ax    ;2
    push  bx    ;3
; print bl
    mov   ax,bx
    cmp   al,'0'
    je    ricoh_bl_0
    call  print_byte    ; bl
    pop   ax    ; bx
    jmp   ricoh_bl_no0
ricoh_bl_0:
; print bh
    pop ax    ;bx is back
    cmp   ah,'0'
    je    ricoh_bh_0
ricoh_bl_no0:
    xchg  al,ah
    call  print_byte    ; bh
    pop   ax   ; original  ax
    jmp   ricoh_bh_no0
ricoh_bh_0:
; print al
    pop ax    ; real ax
    cmp   al,'0'
    je    ricoh_al_0
ricoh_bh_no0:
    push  ax
    call  print_byte    ; al
    pop   ax
    jmp   ricoh_al_n0
ricoh_al_0:
    cmp   ah,'0'
    je    ricoh_ah_0
ricoh_al_n0:
    xchg  al,ah
    call  print_byte    ;ah
; here ah has been printed
    pop   ax    ; pop out dx
    jmp   ricoh_ah_n0
ricoh_ah_0:
    pop   ax
    cmp   al,'0'
    je    ricoh_dl_0
ricoh_ah_n0:
    push  ax
    call  print_byte    ;dl
    pop   ax
ricoh_dl_0:
    xchg  al,ah
    call  print_byte    ;dh
send_comma:
    mov   al,delimiter
    call  print_byte
    ret
ricoh_ret:
    ret
;
; HP LaserJet
;
init_hp:
; set up escape codes to enter raster graphics
; determine resolution
    push  si
    push  di
    mov   word ptr hp_res,'03'
    mov   word ptr hp_res+2,'R0'
    cmp   pdensity,3
    je    send_hp_init
    mov   word ptr hp_res,'51'
    mov   word ptr hp_res+2,'R0'
    cmp   pdensity,2
    je    send_hp_init
    mov   word ptr hp_res,'01'
    mov   word ptr hp_res+2,'R0'
    cmp   pdensity,1
    je    send_hp_init
    mov   word ptr hp_res,'57'
    mov   hp_res+2,'R'
    mov   hp_res+3,0ffh
send_hp_init:
    lea    si,hp_init
    call    print_out
    lea    si,hp_start
    call    print_out
; convert the bytes (in bp) to decimal
    mov    ax,width_bytes
    mov    dx,0
    call   convert_to_decimal
;    dh has 1's, dl has 10's, ah has 100's, al has 1000's
    cld
    lea   di,hp_count
    cmp   al,'0'
    jne   do_hp_1000
    cmp   ah,'0'
    je    no_hp_100
do_hp_1000:
    mov   ds:[di],al
    inc  di
    xchg  al,ah
    mov   ds:[di],al
    inc  di
no_hp_100:
    xchg  ax,dx
    mov   ds:[di],al
    inc  di
    xchg  al,ah
    mov   ds:[di],al
    inc  di
    mov   al,'W'
    mov   ds:[di],al
    inc  di
    mov   al,0ffh
    mov   ds:[di],al
    pop   di
    pop   si
    ret
;
init_p24:
    push    cx
    push    dx
    push    si
    push    di
    lea    si,p24_init
    call    print_out
; we have to set up the columns to print out
; and the density
; bp has the columns to print
    mov   ax,width_bytes
    mov   cx,3
    shl   ax,cl
    mul   cx
    inc   ax
    mov   p24_count,ax
; now density
    mov   al,pdensity
    add   al,8
    cmp   al,10
    jl     p24_10
    inc   al
p24_10:
    cmp   al,12
    jle   p24_12
    mov   al,12
p24_12:
    mov   p24_density,al
    pop   di
    pop   si
    pop   dx
    pop   cx
    ret
;
init_lq:
    push    cx
    push    dx
    push    si
    push    di
    lea    si,lq_init
    call    print_out
; we have to set up the columns to print out
; and the density
; bp has the columns to print
    mov   ax,width_bytes
    mov   cx,3
    shl   ax,cl
    mov   lq_count,ax
; now density
    mov   al,pdensity
    add   al,32
    cmp   al,34
    jl     lq_12
    add   al,4
lq_10:
    cmp   al,40
    jle   lq_12
    mov   al,40
lq_12:
    mov   lq_density,al
    pop   di
    pop   si
    pop   dx
    pop   cx
    ret

init_bj80:
; here  have to print esc,A,9 to set eight lines per inch
; or we have to use the esc3 to set lines per inch
    push    cx
    push    dx
    push    si
    push    di
    lea    si,esc3
    call    print_out
; we have to set up the columns to print out
; and the density
; bp has the columns to print
    mov   ax,width_bytes
    mov   cx,3
    shl   ax,cl
    mov   escm_lines,ax
    mov   esck_lines,ax
; now density
    mov   al,pdensity
    mov   escm_density,al
    jmp   esck_setup

init_tosh:
; here  have to print esc,A,9 to set eight lines per inch
; or we have to use the esc3 to set lines per inch
    push    cx
    push    dx
    push    si
    push    di
    lea    si,tosh_init
    call    print_out
; set up density
    mov    tosh_den,29
    cmp    pdensity,1
    jg    no_tosh_den
    mov    tosh_den,';'
no_tosh_den:
; initialize toshiba columns to 0
    mov    ax,width_bytes
    mov   cl,3
    shl   ax,cl
    mov   dx,0
    call   convert_to_decimal
;    dh has 1's, dl has 10's, ah has 100's, al has 1000's
    lea   di,tosh_count
    cld
    mov   ds:[di],ax
    inc  di
    inc  di
    mov  ax,dx
    mov   ds:[di],ax
    pop   di
    pop   si
    pop   dx
    pop   cx
    ret

init_esck:
; here  have to print esc,A,9 to set eight lines per inch
; or we have to use the esc3 to set lines per inch
    push    cx
    push    dx
    push    si
    push    di
    lea    si,esca9
    call    print_out
; we have to set up the columns to print out
; and the density
; bp has the columns to print
    mov   ax,width_bytes
    mov   cx,3
    shl   ax,cl
    mov   escm_lines,ax
    mov   esck_lines,ax
    mov   nec_width,ax
; now density
    mov   al,pdensity
    mov   escm_density,al
esck_setup:
; for ESC_K printers
    mov   esck_density,'K'
    cmp   al,0
    je    done_density
    mov   esck_density,'L'
    cmp   al,1
    je    done_density
    mov   esck_density,'Z'
    cmp   al,2
    je    done_density
done_density:
    pop   di
    pop   si
    pop   dx
    pop   cx
    ret

;
init_oki:
; this initializes the OKI by issuing the 03 character
;
    push    si
    lea    si,oki_start
    call    print_out
    pop    si
    ret

;
;========================================;
;    ;
;   LINE ROUTINES    ;
;    ;
;========================================;
; this is the line printing setup
; performed once at the head of each line
;
line_cent:
    push    si
    lea    si,cent_line
line_exit:
    call    print_out
    pop    si
    ret
;
;
;
line_gem:
    push    si
    lea    si,gem_line
   jmp  line_exit
;
;
line_hp:
    push    si
    lea    si,hp_transfer
   jmp  line_exit
line_tosh:
    push    si
    lea    si,tosh_line
   jmp  line_exit

line_nec:
    push    si
    lea    si,nec_line
   jmp  line_exit

line_esck:
    push    si
    lea    si,esck
   jmp  line_exit


line_p24:
    push    si
    lea    si,p24_line
   jmp  line_exit

line_lq:
    push    si
    lea    si,lq_line
   jmp  line_exit

line_ncr:
    push    si
    lea    si,ncr_gr
   jmp  line_exit

line_bj80:
    push    si
    lea    si,escm
   jmp  line_exit

;
line_oki:
; this issues the oki crlf
    push    si
    lea    si,oki_crlf
   jmp  line_exit

;
end_hp:
    push    si
    lea    si,hp_end
   jmp  line_exit

end_cent:
    push    si
    lea    si,cent_stop
   jmp  line_exit

;
end_oki:
    push    si
    lea    si,oki_end
   jmp  line_exit

;
CSEG    ENDS
    end
